@@ -101,10 +101,11 @@ struct ares_hosts_entry {
 static ares_status_t ares__read_file_into_buf(const char  *filename,
                                               ares__buf_t *buf)
 {
-  FILE          *fp      = NULL;
-  unsigned char *ptr     = NULL;
-  size_t         len     = 0;
-  size_t         ptr_len = 0;
+  FILE          *fp        = NULL;
+  unsigned char *ptr       = NULL;
+  size_t         len       = 0;
+  size_t         ptr_len   = 0;
+  long           ftell_len = 0;
   ares_status_t  status;
 
   if (filename == NULL || buf == NULL) {
@@ -133,7 +134,14 @@ static ares_status_t ares__read_file_into_buf(const char  *filename,
     status = ARES_EFILE;
     goto done;
   }
-  len = (size_t)ftell(fp);
+
+  ftell_len = ftell(fp);
+  if (ftell_len < 0) {
+    status = ARES_EFILE;
+    goto done;
+  }
+  len = (size_t)ftell_len;
+
   if (fseek(fp, 0, SEEK_SET) != 0) {
     status = ARES_EFILE;
     goto done;
@@ -305,81 +313,51 @@ fail:
   return NULL;
 }
 
-static ares_bool_t ares__hosts_entry_ipaddr_exists(ares_hosts_entry_t *entry,
-                                                   const char         *ipaddr)
+typedef enum {
+  ARES_MATCH_NONE   = 0,
+  ARES_MATCH_IPADDR = 1,
+  ARES_MATCH_HOST   = 2
+} ares_hosts_file_match_t;
+
+static ares_status_t ares__hosts_file_merge_entry(ares_hosts_file_t  *hf,
+                                                  ares_hosts_entry_t *existing,
+                                                  ares_hosts_entry_t *entry,
+                                                  ares_hosts_file_match_t matchtype)
 {
   ares__llist_node_t *node;
 
-  for (node = ares__llist_node_first(entry->ips); node != NULL;
-       node = ares__llist_node_next(node)) {
-    const char *myaddr = ares__llist_node_val(node);
-    if (strcmp(myaddr, ipaddr) == 0) {
-      return ARES_TRUE;
-    }
-  }
+  /* If we matched on IP address, we know there can only be 1, so there's no
+   * reason to do anything */
+  if (matchtype != ARES_MATCH_IPADDR) {
+    while ((node = ares__llist_node_first(entry->ips)) != NULL) {
+      const char *ipaddr = ares__llist_node_val(node);
 
-  return ARES_FALSE;
-}
+      if (ares__htable_strvp_get_direct(hf->iphash, ipaddr) != NULL) {
+        ares__llist_node_destroy(node);
+        continue;
+      }
 
-static ares_bool_t ares__hosts_entry_host_exists(ares_hosts_entry_t *entry,
-                                                 const char         *host)
-{
-  ares__llist_node_t *node;
-
-  for (node = ares__llist_node_first(entry->ips); node != NULL;
-       node = ares__llist_node_next(node)) {
-    const char *myhost = ares__llist_node_val(node);
-    if (strcasecmp(myhost, host) == 0) {
-      return ARES_TRUE;
-    }
-  }
-
-  return ARES_FALSE;
-}
-
-static ares_status_t ares__hosts_file_merge_entry(ares_hosts_entry_t *existing,
-                                                  ares_hosts_entry_t *entry)
-{
-  ares__llist_node_t *node;
-
-  while ((node = ares__llist_node_first(entry->ips)) != NULL) {
-    char *ipaddr = ares__llist_node_claim(node);
-
-    if (ares__hosts_entry_ipaddr_exists(existing, ipaddr)) {
-      ares_free(ipaddr);
-      continue;
-    }
-
-    if (ares__llist_insert_last(existing->ips, ipaddr) == NULL) {
-      ares_free(ipaddr);
-      return ARES_ENOMEM;
+      ares__llist_node_move_parent_last(node, existing->ips);
     }
   }
 
 
   while ((node = ares__llist_node_first(entry->hosts)) != NULL) {
-    char *hostname = ares__llist_node_claim(node);
+    const char *hostname = ares__llist_node_val(node);
 
-    if (ares__hosts_entry_host_exists(existing, hostname)) {
-      ares_free(hostname);
+    if (ares__htable_strvp_get_direct(hf->hosthash, hostname) != NULL) {
+      ares__llist_node_destroy(node);
       continue;
     }
 
-    if (ares__llist_insert_last(existing->hosts, hostname) == NULL) {
-      ares_free(hostname);
-      return ARES_ENOMEM;
-    }
+    ares__llist_node_move_parent_last(node, existing->hosts);
   }
 
   ares__hosts_entry_destroy(entry);
   return ARES_SUCCESS;
 }
 
-typedef enum {
-  ARES_MATCH_NONE   = 0,
-  ARES_MATCH_IPADDR = 1,
-  ARES_MATCH_HOST   = 2
-} ares_hosts_file_match_t;
+
 
 static ares_hosts_file_match_t
   ares__hosts_file_match(const ares_hosts_file_t *hf, ares_hosts_entry_t *entry,
@@ -414,14 +392,20 @@ static ares_status_t ares__hosts_file_add(ares_hosts_file_t  *hosts,
                                           ares_hosts_entry_t *entry)
 {
   ares_hosts_entry_t     *match  = NULL;
-  ares_status_t           status = ARES_SUCCESS;
+  ares_status_t           status       = ARES_SUCCESS;
   ares__llist_node_t     *node;
   ares_hosts_file_match_t matchtype;
+  size_t                  num_hostnames;
+
+  /* Record the number of hostnames in this entry file.  If we merge into an
+   * existing record, these will be *appended* to the entry, so we'll count
+   * backwards when adding to the hosts hashtable */
+  num_hostnames = ares__llist_len(entry->hosts);
 
   matchtype = ares__hosts_file_match(hosts, entry, &match);
 
   if (matchtype != ARES_MATCH_NONE) {
-    status = ares__hosts_file_merge_entry(match, entry);
+    status = ares__hosts_file_merge_entry(hosts, match, entry, matchtype);
     if (status != ARES_SUCCESS) {
       ares__hosts_entry_destroy(entry);
       return status;
@@ -442,10 +426,17 @@ static ares_status_t ares__hosts_file_add(ares_hosts_file_t  *hosts,
     }
   }
 
-  for (node = ares__llist_node_first(entry->hosts); node != NULL;
-       node = ares__llist_node_next(node)) {
+  /* Go backwards, on a merge, hostnames are appended.  Breakout once we've
+   * consumed all the hosts that we appended */
+  for (node = ares__llist_node_last(entry->hosts); node != NULL;
+       node = ares__llist_node_prev(node)) {
     const char *val = ares__llist_node_val(node);
 
+    if (num_hostnames == 0)
+      break;
+
+    num_hostnames--;
+
     /* first hostname match wins.  If we detect a duplicate hostname for another
      * ip it will automatically be added to the same entry */
     if (ares__htable_strvp_get(hosts->hosthash, val, NULL)) {
@@ -460,6 +451,22 @@ static ares_status_t ares__hosts_file_add(ares_hosts_file_t  *hosts,
   return ARES_SUCCESS;
 }
 
+static ares_bool_t ares__hosts_entry_isdup(ares_hosts_entry_t *entry,
+                                           const char         *host)
+{
+  ares__llist_node_t *node;
+
+  for (node = ares__llist_node_first(entry->ips); node != NULL;
+       node = ares__llist_node_next(node)) {
+    const char *myhost = ares__llist_node_val(node);
+    if (strcasecmp(myhost, host) == 0) {
+      return ARES_TRUE;
+    }
+  }
+
+  return ARES_FALSE;
+}
+
 static ares_status_t ares__parse_hosts_hostnames(ares__buf_t        *buf,
                                                  ares_hosts_entry_t *entry)
 {
@@ -510,7 +517,7 @@ static ares_status_t ares__parse_hosts_hostnames(ares__buf_t        *buf,
     }
 
     /* Don't add a duplicate to the same entry */
-    if (ares__hosts_entry_host_exists(entry, hostname)) {
+    if (ares__hosts_entry_isdup(entry, hostname)) {
       continue;
     }
 
@@ -942,8 +949,14 @@ ares_status_t ares__hosts_entry_to_hostent(const ares_hosts_entry_t *entry,
 
   /* Copy aliases */
   naliases = ares__llist_len(entry->hosts) - 1;
+
+  /* Cap at 100, some people use https://github.com/StevenBlack/hosts and we
+   * don't need 200k+ aliases */
+  if (naliases > 100)
+    naliases = 100;
+
   (*hostent)->h_aliases =
-    ares_malloc_zero((naliases + 1) * sizeof((*hostent)->h_aliases));
+    ares_malloc_zero((naliases + 1) * sizeof(*(*hostent)->h_aliases));
   if ((*hostent)->h_aliases == NULL) {
     status = ARES_ENOMEM;
     goto fail;
@@ -960,6 +973,10 @@ ares_status_t ares__hosts_entry_to_hostent(const ares_hosts_entry_t *entry,
       goto fail;
     }
     idx++;
+
+    /* Break out if artificially capped */
+    if (idx == naliases)
+      break;
     node = ares__llist_node_next(node);
   }
 
@@ -980,6 +997,7 @@ static ares_status_t
   const char                 *primaryhost;
   ares__llist_node_t         *node;
   ares_status_t               status;
+  size_t                      cnt = 0;
 
   node        = ares__llist_node_first(entry->hosts);
   primaryhost = ares__llist_node_val(node);
@@ -989,6 +1007,12 @@ static ares_status_t
   while (node != NULL) {
     const char *host = ares__llist_node_val(node);
 
+    /* Cap at 100 entries. , some people use https://github.com/StevenBlack/hosts
+     * and we don't need 200k+ aliases */
+    cnt++;
+    if (cnt > 100)
+      break;
+
     cname = ares__append_addrinfo_cname(&cnames);
     if (cname == NULL) {
       status = ARES_ENOMEM;
