@@ -54,18 +54,12 @@ typedef struct ares_rand_rc4 {
 } ares_rand_rc4;
 
 
-#  ifdef _MSC_VER
-typedef unsigned __int64 cares_u64;
-#  else
-typedef unsigned long long cares_u64;
-#  endif
-
 
 static unsigned int ares_u32_from_ptr(void *addr)
 {
   if (sizeof(void *) == 8) {
-    return (unsigned int)((((cares_u64)addr >> 32) & 0xFFFFFFFF) |
-                          ((cares_u64)addr & 0xFFFFFFFF));
+    return (unsigned int)((((ares_uint64_t)addr >> 32) & 0xFFFFFFFF) |
+                          ((ares_uint64_t)addr & 0xFFFFFFFF));
   }
   return (unsigned int)((size_t)addr & 0xFFFFFFFF);
 }
