@@ -91,7 +91,7 @@ static ares_bool_t ares_ipv6_subnet_matches(const unsigned char  netbase[16],
   memset(mask, 0xFF, netmask / 8);
 
   /* Set remaining bits */
-  if (netmask % 8) {
+  if (netmask % 8 && netmask < 128 /* Silence coverity */) {
     mask[netmask / 8] = (unsigned char)(0xff << (8 - (netmask % 8)));
   }
 
