@@ -45,9 +45,7 @@ TEST_F(DefaultChannelTest, GetServersFailures) {
 }
 
 TEST_F(DefaultChannelTest, SetServers) {
-  EXPECT_EQ(ARES_SUCCESS, ares_set_servers(channel_, nullptr));
-  std::vector<std::string> empty;
-  EXPECT_EQ(empty, GetNameServers(channel_));
+  EXPECT_EQ(ARES_ENODATA, ares_set_servers(channel_, nullptr));
 
   struct ares_addr_node server1;
   struct ares_addr_node server2;
@@ -60,14 +58,12 @@ TEST_F(DefaultChannelTest, SetServers) {
   EXPECT_EQ(ARES_ENODATA, ares_set_servers(nullptr, &server1));
 
   EXPECT_EQ(ARES_SUCCESS, ares_set_servers(channel_, &server1));
-  std::vector<std::string> expected = {"1.2.3.4", "2.3.4.5"};
+  std::vector<std::string> expected = {"1.2.3.4:53", "2.3.4.5:53"};
   EXPECT_EQ(expected, GetNameServers(channel_));
 }
 
 TEST_F(DefaultChannelTest, SetServersPorts) {
-  EXPECT_EQ(ARES_SUCCESS, ares_set_servers_ports(channel_, nullptr));
-  std::vector<std::string> empty;
-  EXPECT_EQ(empty, GetNameServers(channel_));
+  EXPECT_EQ(ARES_ENODATA, ares_set_servers_ports(channel_, nullptr));
 
   struct ares_addr_port_node server1;
   struct ares_addr_port_node server2;
@@ -84,7 +80,7 @@ TEST_F(DefaultChannelTest, SetServersPorts) {
   EXPECT_EQ(ARES_ENODATA, ares_set_servers_ports(nullptr, &server1));
 
   EXPECT_EQ(ARES_SUCCESS, ares_set_servers_ports(channel_, &server1));
-  std::vector<std::string> expected = {"1.2.3.4:111", "2.3.4.5"};
+  std::vector<std::string> expected = {"1.2.3.4:111", "2.3.4.5:53"};
   EXPECT_EQ(expected, GetNameServers(channel_));
 }
 
@@ -97,12 +93,12 @@ TEST_F(DefaultChannelTest, SetServersCSV) {
 
   EXPECT_EQ(ARES_SUCCESS,
             ares_set_servers_csv(channel_, "1.2.3.4,0102:0304:0506:0708:0910:1112:1314:1516,2.3.4.5"));
-  std::vector<std::string> expected = {"1.2.3.4", "0102:0304:0506:0708:0910:1112:1314:1516", "2.3.4.5"};
+  std::vector<std::string> expected = {"1.2.3.4:53", "[0102:0304:0506:0708:0910:1112:1314:1516]:53", "2.3.4.5:53"};
   EXPECT_EQ(expected, GetNameServers(channel_));
 
   // Same, with spaces
   EXPECT_EQ(ARES_EBADSTR,
-            ares_set_servers_csv(channel_, "1.2.3.4 , 0102:0304:0506:0708:0910:1112:1314:1516, 2.3.4.5"));
+            ares_set_servers_csv(channel_, "1.2.3.4 , [0102:0304:0506:0708:0910:1112:1314:1516]:53, 2.3.4.5"));
 
   // Same, with ports
   EXPECT_EQ(ARES_SUCCESS,
@@ -114,7 +110,7 @@ TEST_F(DefaultChannelTest, SetServersCSV) {
   EXPECT_EQ(expected2, GetNameServers(channel_));
 
   // Should survive duplication
-  ares_channel channel2;
+  ares_channel_t *channel2;
   EXPECT_EQ(ARES_SUCCESS, ares_dup(&channel2, channel_));
   EXPECT_EQ(expected2, GetNameServers(channel2));
   ares_destroy(channel2);
@@ -126,11 +122,6 @@ TEST_F(DefaultChannelTest, SetServersCSV) {
               ares_set_servers_csv(channel_, "1.2.3.4,0102:0304:0506:0708:0910:1112:1314:1516,2.3.4.5"));
   }
 
-  // Blank servers
-  EXPECT_EQ(ARES_SUCCESS, ares_set_servers_csv(channel_, ""));
-  std::vector<std::string> none;
-  EXPECT_EQ(none, GetNameServers(channel_));
-
   EXPECT_EQ(ARES_EBADSTR, ares_set_servers_csv(channel_, "2.3.4.5,1.2.3.4:,3.4.5.6"));
   EXPECT_EQ(ARES_EBADSTR, ares_set_servers_csv(channel_, "2.3.4.5,1.2.3.4:Z,3.4.5.6"));
 }
@@ -197,15 +188,17 @@ TEST_F(LibraryTest, Mkquery) {
 TEST_F(LibraryTest, CreateQuery) {
   byte* p;
   int len;
+  // This is hard to really test with escaping since DNS names don't allow
+  // bad characters.  So we'll escape good characters.
   EXPECT_EQ(ARES_SUCCESS,
-            ares_create_query("exam\\@le.com", C_IN, T_A, 0x1234, 0,
+            ares_create_query("ex\\097m\\ple.com", C_IN, T_A, 0x1234, 0,
                               &p, &len, 0));
   std::vector<byte> data(p, p + len);
   ares_free_string(p);
 
   std::string actual = PacketToString(data);
   DNSPacket pkt;
-  pkt.set_qid(0x1234).add_question(new DNSQuestion("exam@le.com", T_A));
+  pkt.set_qid(0x1234).add_question(new DNSQuestion("example.com", T_A));
   std::string expected = PacketToString(pkt.data());
   EXPECT_EQ(expected, actual);
 }
